<?php

namespace App\Http\Controllers\API;

use App\Http\Requests\API\CreateServersAPIRequest;
use App\Http\Requests\API\UpdateServersAPIRequest;
use App\Models\Servers;
use App\Repositories\ServersRepository;
use Illuminate\Http\Request;
use App\Http\Controllers\AppBaseController;
use InfyOm\Generator\Criteria\LimitOffsetCriteria;
use Prettus\Repository\Criteria\RequestCriteria;
use Response;

/**
 * Class ServersController
 * @package App\Http\Controllers\API
 */

class ServersAPIController extends AppBaseController
{
    /** @var  ServersRepository */
    private $serversRepository;

    public function __construct(ServersRepository $serversRepo)
    {
        $this->serversRepository = $serversRepo;
    }

    /**
     * @param Request $request
     * @return Response
     *
     * @SWG\Get(
     *      path="/servers",
     *      summary="Get a listing of the Servers.",
     *      tags={"Servers"},
     *      description="Get all Servers",
     *      produces={"application/json"},
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="array",
     *                  @SWG\Items(ref="#/definitions/Servers")
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function index(Request $request)
    {
        $this->serversRepository->pushCriteria(new RequestCriteria($request));
        $this->serversRepository->pushCriteria(new LimitOffsetCriteria($request));
        $servers = $this->serversRepository->all();

        return $this->sendResponse($servers->toArray(), 'Сервера успешно получены');
    }

    /**
     * @param CreateServersAPIRequest $request
     * @return Response
     *
     * @SWG\Post(
     *      path="/servers",
     *      summary="Store a newly created Servers in storage",
     *      tags={"Servers"},
     *      description="Store Servers",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Servers that should be stored",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Servers")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Servers"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function store(CreateServersAPIRequest $request)
    {
        $input = $request->all();

        $servers = $this->serversRepository->create($input);

        return $this->sendResponse($servers->toArray(), 'Сервер успешно добавлен');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Get(
     *      path="/servers/{id}",
     *      summary="Display the specified Servers",
     *      tags={"Servers"},
     *      description="Get Servers",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Servers",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Servers"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function show($id)
    {
        /** @var Servers $servers */
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            return $this->sendError('Сервер не найден');
        }

        return $this->sendResponse($servers->toArray(), 'Сервер успешно получен');
    }

    /**
     * @param int $id
     * @param UpdateServersAPIRequest $request
     * @return Response
     *
     * @SWG\Put(
     *      path="/servers/{id}",
     *      summary="Update the specified Servers in storage",
     *      tags={"Servers"},
     *      description="Update Servers",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Servers",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Parameter(
     *          name="body",
     *          in="body",
     *          description="Servers that should be updated",
     *          required=false,
     *          @SWG\Schema(ref="#/definitions/Servers")
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  ref="#/definitions/Servers"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function update($id, UpdateServersAPIRequest $request)
    {
        $input = $request->all();

        /** @var Servers $servers */
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            return $this->sendError('Сервер не найден');
        }

        $servers = $this->serversRepository->update($input, $id);

        return $this->sendResponse($servers->toArray(), 'Сервер успешно сохранен');
    }

    /**
     * @param int $id
     * @return Response
     *
     * @SWG\Delete(
     *      path="/servers/{id}",
     *      summary="Remove the specified Servers from storage",
     *      tags={"Servers"},
     *      description="Delete Servers",
     *      produces={"application/json"},
     *      @SWG\Parameter(
     *          name="id",
     *          description="id of Servers",
     *          type="integer",
     *          required=true,
     *          in="path"
     *      ),
     *      @SWG\Response(
     *          response=200,
     *          description="successful operation",
     *          @SWG\Schema(
     *              type="object",
     *              @SWG\Property(
     *                  property="success",
     *                  type="boolean"
     *              ),
     *              @SWG\Property(
     *                  property="data",
     *                  type="string"
     *              ),
     *              @SWG\Property(
     *                  property="message",
     *                  type="string"
     *              )
     *          )
     *      )
     * )
     */
    public function destroy($id)
    {
        /** @var Servers $servers */
        $servers = $this->serversRepository->findWithoutFail($id);

        if (empty($servers)) {
            return $this->sendError('Сервер не найден');
        }

        $servers->delete();

        return $this->sendResponse($id, 'Сервер успешно удален');
    }
}
