<?php

namespace App\Http\Controllers;

use App\Console\Commands\BaseCommand;
use App\DataTables\WaybillsDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateWaybillsRequest;
use App\Http\Requests\UpdateWaybillsRequest;
use App\Repositories\WaybillsRepository;
use Carbon\Carbon;
use Flash;
use Illuminate\Http\Request;
use Response;

class WaybillsController extends AppBaseController
{
    /** @var  WaybillsRepository */
    private $waybillsRepository;

    public function __construct(WaybillsRepository $waybillsRepo)
    {
        $this->waybillsRepository = $waybillsRepo;
    }

    /**
     * Display a listing of the Waybills.
     *
     * @param WaybillsDataTable $waybillsDataTable
     * @return \Illuminate\Http\JsonResponse|\Illuminate\View\View
     */
    public function index(WaybillsDataTable $waybillsDataTable)
    {
        return $waybillsDataTable->render('waybills.index');
    }

    /**
     * Show the form for creating a new Waybills.
     *
     * @return Response
     */
    public function create()
    {
        return view('waybills.create');
    }

    /**
     * Store a newly created Waybills in storage.
     *
     * @param CreateWaybillsRequest $request
     *
     * @return \Illuminate\Http\RedirectResponse
     */
    public function store(CreateWaybillsRequest $request)
    {
        $array = $request->except('_token');

        $array['start_date'] = new Carbon($array['start_date']);
        $array['end_date']   = new Carbon($array['end_date']);

        if ((int)$array['owners_id'] === 0) {
            unset($array['owners_id']);
        }

        if ((int)$array['drivers_id'] === 0) {
            unset($array['drivers_id']);
        }

        if ((int)$array['trailer'] === 0) {
            unset($array['trailer']);
        }

        $waybill = $this->waybillsRepository->create($array);

        if (!$waybill) {
            Flash::error('Ошибка генерации путевого листа.');

            return redirect()->back()->withInput();
        }

        $exitCode = \Artisan::call('waybill:create', ['id' => $waybill->id, 'type' => $waybill->type]);

        if ($this->getValueExitCode(BaseCommand::class, $exitCode) === 'SUCCESS') {
            Flash::info('Путевой лист успешно сгенерирован, загрузка файла начнется автоматически.');

            \Session::flash('download.in.the.next.request', route('waybills.file', $waybill->id));

            return redirect()->route('waybills.show', $waybill->id);
        }

        Flash::info('Путевой лист успешно сохранен, но файл сгенерировать не удалось.');

        return redirect()->route('waybills.show', $waybill->id);
    }

    /**
     * Display the specified Waybills.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $waybills = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybills)) {
            Flash::error('Путевой лист не найден');

            return redirect(route('waybills.index'));
        }

        return view('waybills.show')->with('waybills', $waybills);
    }

    /**
     * Download waybill file
     *
     * @param  int $id
     *
     * @param Request $request
     * @return Response
     */
    public function file($id, Request $request)
    {
        $waybill = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybill)) {
            Flash::error('Путевой лист не найден');

            return redirect(route('waybills.index'));
        }

        return response()->download(resource_path($waybill->file));
    }

    /**
     * Show the form for editing the specified Waybills.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $waybills = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybills)) {
            Flash::error('Путевой лист не найден');

            return redirect(route('waybills.index'));
        }

        return view('waybills.edit')->with('waybills', $waybills);
    }

    /**
     * Update the specified Waybills in storage.
     *
     * @param  int $id
     * @param UpdateWaybillsRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateWaybillsRequest $request)
    {
        $waybills = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybills)) {
            Flash::error('Путевой лист не найден');

            return redirect(route('waybills.index'));
        }

        $waybills = $this->waybillsRepository->update($request->all(), $id);

        Flash::success('Путевой лист успешно отредактирован.');

        return redirect(route('waybills.index'));
    }

    /**
     * Remove the specified Waybills from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $waybill = $this->waybillsRepository->findWithoutFail($id);

        if (empty($waybill)) {
            Flash::error('Путевой лист не найден');

            return redirect(route('waybills.index'));
        }

        @unlink(resource_path($waybill->file));

        $this->waybillsRepository->delete($id);

        Flash::success('Путевой лист успешно удален.');

        return redirect(route('waybills.index'));
    }
}
